#!/usr/bin/env python

import os
import sys
from math import *
from copy import copy

def set_val(d, x, y, val):
	d[(x,y)] = val
	return
	if not d.has_key(x):
		d[x] = {}
	d[x][y] = val

def display(data):

	d = data.keys()
	d.sort()
	for x, y in d:
		print x, y, data[x,y]

def write_file(fname, data, max_x, max_y, mul=1, repeat=0, avg=0):

	f = open(fname, "w")
	for x in range(max_x):
		for y in range(max_y):
			if data.has_key((x,y)):
				for m in range(mul):
					for n in range(mul):
						f.write("%f\t%f\t%f\n" % (x+m*max_x,
											y+n*max_y, data[x, y]))
	if repeat:
		for m in range(-repeat,repeat+mul*max_x+1):
			for n in range(-repeat,repeat+mul*max_y+1):
				if 0 <= m and m <= mul*max_x and 0 <= n and n <= mul*max_y:
					continue
				f.write("%f\t%f\t%f\n" % (m, n, avg))
		
	f.close()

def select_col(d, col, max_x, max_y):

	r = {}
	for y in range(max_y+1):
		if d.has_key((col,y)):
			r[y] = d[col, y]
	return r

def select_row(d, row, max_x, max_y):

	r = {}
	for x in range(max_x+1):
		if d.has_key((x,row)):
			r[x] = d[x, row]
	return r

def by_val(kv1, kv2):
	k1, v1 = kv1
	k2, v2 = kv2
	return cmp(v1, v2)

def by_key(kv1, kv2):
	k1, v1 = kv1
	k2, v2 = kv2
	return cmp(k1, k2)

def sorted_index(d):
	l = d.items()
	l.sort(by_val)
	return map(lambda (x,y): x, l)

def reorder_cols(d, i0, max_x, max_y):
	res = {}
	idx = {}

	isort = copy(i0)
	isort.sort()

	for x in range(len(i0)):
		idx[isort.pop(0)] = i0.pop(0)

	for x in range(max_x+1):
		for (y, idx_y) in idx.items():
			if d.has_key((x, idx_y)):
				res[x,y] = d[x, idx[y]]
	return res

def reorder_rows(d, i0, max_x, max_y):
	res = {}
	idx = {}

	isort = copy(i0)
	isort.sort()

	for x in range(len(i0)):
		idx[isort.pop(0)] = i0.pop(0)

	for y in range(max_y+1):
		for (x, idx_x) in idx.items():
			if d.has_key((idx_x, y)):
				res[x,y] = d[idx[x], y]
	return res

def read_file(fname):

	data = {}
	max_x = 0
	max_y = 0
	f = open(fname)
	for l in f.readlines():
		l = l.strip().split('\t')
		l = map(lambda x: x[:-1], l)
		l = map(float, l)
		x = int(l[0])
		y = int(l[1])
		set_val(data, x, y, float(l[2]))
		max_x = max(x, max_x)
		max_y = max(y, max_y)

	return data, max_x, max_y

def std_dev(data, max_x, max_y ):

	avg = dot_average(data, max_x, max_y)

	newd = {}

	for x in range(0, max_x+1):
		for y in range(0, max_y+1):
			if not data.has_key((x, y)):
				continue
			dv = data[x, y] - avg 
			new_val = sqrt(dv * dv)
			set_val(newd, x, y, new_val)

	return newd
	
def remap(data, max_x, max_y , bitstep_x, bitstep_y):

	newd = {}

	x = 0

	for step_x in range(bitstep_x):
		for new_x in range(step_x, max_x+1, bitstep_x):

			y = 0

			for step_y in range(bitstep_y):
				for new_y in range(step_y, max_y+1, bitstep_y):

					set_val(newd, new_x, new_y, data[x, y])

					y += 1
		
			x += 1

	return newd
	
def average(data, max_x, max_y, block_x, block_y):

	newd = {}
	blocksize = block_x * block_y

	for new_x in range(0, max_x+1, block_x):
		for new_y in range(0, max_y+1, block_y):

			total = 0.0

			for step_x in range(block_x):
				for step_y in range(block_y):

					total += data[new_x+step_x, new_y+step_y]

			set_val(newd, new_x, new_y, total / blocksize)

	return newd

def weightaverage(data, max_x, max_y, block_x, block_y):

	newd = {}
	blocksize = block_x * block_y

	range_total = 0.0
	for step_x in range(-block_x, block_x+1):
		for step_y in range(-block_y, block_y+1):

			sqdist = (step_x*step_x) + (step_y*step_y)
			rdist = block_x * block_y / (20.0+sqdist)
			range_total += rdist

			print step_x, step_y, rdist

	print range_total

	for new_x in range(0, max_x+1):
		for new_y in range(0, max_y+1):

			total = 0.0

			for step_x in range(-block_x, block_x+1):
				for step_y in range(-block_y, block_y+1):

					sqdist = (step_x*step_x) + (step_y*step_y)
					dist = block_x * block_y / (1+sqdist)

					nx = (new_x+step_x+max_y) % max_x
					ny = (new_y+step_y+max_y) % max_y
					total += data[nx, ny] * dist / range_total

			set_val(newd, new_x, new_y, total / blocksize)

	return newd

def dot_average(d, max_x, max_y):

	total = 0.0
	num = 0.0
	for x in range(0, max_x+1):
		for y in range(0, max_y+1):
			if d.has_key((x,y)):
				total += d[x,y]
				num += 1
	return total / num

	
os.chdir("graphs")

#fname = "rjd.71680.txt"
fname = "serpent-pval.256000.txt"
#fname = "random.txt"
#fname = "rjd.256000.txt"
#fname = "rjd.256000.txt"
#fname = "rjd-pval.255872.txt"
fname = "cast.20480.txt"
#fname = "rjd.2.5120.txt"
#fname = "rjd.5.256000.txt"
bitstep_x = 2
bitstep_y = 2

d, max_x, max_y = read_file(fname)
#print d

d = remap(d, max_x, max_y, 8, 8)
#d = remap(d, max_x, max_y, 16, 16)
#d = remap(d, max_x, max_y, 32, 32)
#d = remap(d, max_x, max_y, 64, 64)
#d = remap(d, max_x, max_y, 2, 2)
#d = remap(d, max_x, max_y, 4, 4)

d = average(d, max_x, max_y, 8, 8)
d = std_dev(d, max_x, max_y)

c0 = select_col(d, 104, max_x, max_y)
#r0 = select_row(d, 120, max_x, max_y)
i0 = sorted_index(c0)
#i0 = sorted_index(r0)

#d = reorder_rows(d, i0, max_x, max_y)
d = reorder_cols(d, i0, max_x, max_y)

#d = remap(d, max_x, max_y, bitstep_x, bitstep_y)
#d = std_dev(d, max_x, max_y)
#d = average(d, max_x, max_y, bitstep_x, bitstep_y)
#d = weightaverage(d, max_x, max_y, 8, 8)
#display(newd)

avg = dot_average(d, max_x, max_y)

#write_file("remap.%dx%d.%s" % (bitstep_x, bitstep_y, fname), d, max_x, max_y,
#			mul=3,repeat=64, avg=avg)
#write_file("remap.%dx%d.%s" % (bitstep_x, bitstep_y, fname), d, max_x, max_y,
#			repeat=4, avg=avg)
write_file("remap.%dx%d.%s" % (bitstep_x, bitstep_y, fname), d, max_x, max_y)
			

sys.exit(0)

print "creating random file"
from random import Random
r = Random()
d = {}
for x in range(128):
	for y in range(128):
		set_val(d, x, y, 0)

for i in range(256000):
	x = r.randint(0, 127)
	y = r.randint(0, 127)
	rv = r.randint(0, 10)
	set_val(d, x, y, d[(x,y)]+rv)
	if i % 1000 == 0:
		print i

write_file("random.txt", d, 128, 128)
